@@ -1,20 +1,20 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019-2022 A S Lewis
+# Copyright (C) 2019-2023 A S Lewis
 #
-# This program is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation, either version 3 of the License, or (at your option) any later
-# version.
+# This program is free software; you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation; either version 2.1 of the License, or (at your option)
+# any later version.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU General Public License along with
-# this program. If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 
 """FFmpeg manager classes."""
@@ -27,6 +27,7 @@
 # Import other modules
 import os
 import re
+import shutil
 import subprocess
 
 
@@ -95,7 +96,7 @@ class FFmpegManager(object):
             thumbnail_filename (str): Full path to the webp file to be
                 converted to jpg
 
-        Returns:
+        Return values:
 
             False if an attempted conversion fails, or True otherwise
                 (including when no conversion is attempted)
@@ -107,6 +108,13 @@ class FFmpegManager(object):
         or self.app_obj.ffmpeg_fail_flag:
             return True
 
+        # Retain original thumbnails, if required
+        if self.app_obj.ffmpeg_convert_webp_flag \
+        and self.app_obj.ffmpeg_retain_webp_flag:
+            retain_flag = True
+        else:
+            retain_flag = False
+
         # Correct extension for .webp files with the wrong extension
         #   (youtube-dl #25687, #25717)
         _, thumbnail_ext = os.path.splitext(thumbnail_filename)
@@ -123,23 +131,67 @@ class FFmpegManager(object):
                     'webp',
                 )
 
-                os.rename(thumbnail_filename, thumbnail_webp_filename)
+                if not retain_flag:
+                    try:
+                        os.rename(thumbnail_filename, thumbnail_webp_filename)
+                    except:
+                        return False
+                else:
+                    try:
+                        shutil.copyfile(
+                            thumbnail_filename,
+                            thumbnail_webp_filename,
+                        )
+                    except:
+                        return False
+
                 thumbnail_filename = thumbnail_webp_filename
                 thumbnail_ext = 'webp'
 
+            elif thumbnail_ext == 'webp' and \
+            self.is_mislabelled_webp(thumbnail_filename):
+
+                # .jpg mislabelled as .webp (Git #478)
+                thumbnail_jpg_filename = self.replace_extension(
+                    thumbnail_filename,
+                    'jpg',
+                )
+
+                if not retain_flag:
+                    try:
+                        os.rename(thumbnail_filename, thumbnail_jpg_filename)
+                    except:
+                        return False
+                else:
+                    try:
+                        shutil.copyfile(
+                            thumbnail_filename,
+                            thumbnail_jpg_filename,
+                        )
+                    except:
+                        return False
+
+                thumbnail_filename = thumbnail_jpg_filename
+                thumbnail_ext = 'jpg'
+
         # Convert unsupported thumbnail formats to JPEG
         #   (youtube-dl #25687, #25717)
         if thumbnail_ext not in ['jpg', 'png']:
 
             # NB: % is supposed to be escaped with %% but this does not work
-            # for input files so working around with standard substitution
+            #   for input files so working around with standard substitution
             escaped_thumbnail_filename = thumbnail_filename.replace('%', '#')
-            os.rename(thumbnail_filename, escaped_thumbnail_filename)
             escaped_thumbnail_jpg_filename = self.replace_extension(
                 escaped_thumbnail_filename,
                 'jpg',
             )
 
+            # Handle special characters
+            try:
+                os.rename(thumbnail_filename, escaped_thumbnail_filename)
+            except:
+                return False
+
             # Run FFmpeg to convert the thumbnail(s)
             success_flag, msg = self.run_ffmpeg(
                 escaped_thumbnail_filename,
@@ -152,24 +204,35 @@ class FFmpegManager(object):
                 # Conversion failed; most likely because FFmpeg is not
                 #   installed
                 # Rename back to unescaped
-                os.rename(escaped_thumbnail_filename, thumbnail_filename)
+                try:
+                    os.rename(escaped_thumbnail_filename, thumbnail_filename)
+                except:
+                    pass
 
                 return False
 
             else:
 
                 # Conversion succeeded
-                self.app_obj.remove_file(escaped_thumbnail_filename)
+                # Rename the (converted file) to unescaped for further
+                #   processing
                 thumbnail_jpg_filename = self.replace_extension(
                     thumbnail_filename,
                     'jpg',
                 )
 
-                # Rename back to unescaped for further processing
-                os.rename(
-                    escaped_thumbnail_jpg_filename,
-                    thumbnail_jpg_filename
-                )
+                try:
+                    os.rename(
+                        escaped_thumbnail_jpg_filename,
+                        thumbnail_jpg_filename
+                    )
+                except:
+                    return False
+
+                if not retain_flag:
+
+                    # The original .webp file is not retained
+                    self.app_obj.remove_file(escaped_thumbnail_filename)
 
         # Procedure complete
         return True
@@ -187,7 +250,7 @@ class FFmpegManager(object):
 
             path (str): The full path to a file to be processed by FFmpeg
 
-        Returns:
+        Return values:
 
             The modified string
 
@@ -210,7 +273,7 @@ class FFmpegManager(object):
         Returns the path to the FFmpeg executable, which the user may have
         specified themselves. If not, assume ffmpeg is in the system path.
 
-        Returns:
+        Return values:
 
             The path to the executable
 
@@ -224,7 +287,8 @@ class FFmpegManager(object):
 
     def is_webp(self, path):
 
-        """Called by self.convert_webp() and utils.find_thumbnail_webp().
+        """Called by self.convert_webp() and
+        utils.find_thumbnail_webp_intact_or_broken().
 
         Adapted from youtube-dl/youtube-dl/postprocessor/embedthumbnail.py.
 
@@ -240,9 +304,32 @@ class FFmpegManager(object):
         with open(path, 'rb') as fh:
             data = fh.read(12)
 
+        # Test .webp magic number
         return data[0:4] == b'RIFF' and data[8:] == b'WEBP'
 
 
+    def is_mislabelled_webp(self, path):
+
+        """Called by self.convert_webp() and
+        utils.find_thumbnail_webp_intact_or_broken().
+
+        Adapted from self.is_webp().
+
+        Tests whether a file is a .jpg file (perhaps mislabelled as a .webp
+        file), hoping to handle Git #478.
+
+        Args:
+
+            path (str): The full path to a file to be processed by FFmpeg
+
+        """
+
+        with open(path, 'rb') as fh:
+            data = fh.read(3)
+
+        return data[0:3] == b'\xff\xd8\xff'
+
+
     def replace_extension(self, path, ext, expected_real_ext=None):
 
         """Called by self.convert_webp().
@@ -260,7 +347,7 @@ class FFmpegManager(object):
 
             expected_real_ext (str): Not used by Tartube
 
-        Returns:
+        Return values:
 
             The modified path
 
@@ -379,7 +466,7 @@ class FFmpegManager(object):
         stdout, stderr = p.communicate()
         if p.returncode != 0:
             stderr = stderr.decode(utils.get_encoding(), 'replace')
-            return [ False, stderr.strip().split('\n')[-1] ]
+            return [ False, stderr.strip().splitlines()[-1] ]
 
         else:
             return [ self.try_utime(out_path, oldest_mtime, oldest_mtime), '' ]
@@ -427,7 +514,7 @@ class FFmpegManager(object):
         stdout, stderr = p.communicate()
         if p.returncode != 0:
             stderr = stderr.decode(utils.get_encoding(), 'replace')
-            return [ False, stderr.strip().split('\n')[-1] ]
+            return [ False, stderr.strip().splitlines()[-1] ]
 
         else:
             return [ self.try_utime(source_path, mod_time, mod_time), '' ]
@@ -1172,11 +1259,3 @@ class FFmpegOptionsManager(object):
             return source_thumb_path, output_path, return_list
         else:
             return source_video_path, output_path, return_list
-
-
-
-
-
-
-
-
